@@ -102,7 +102,16 @@ class DXWorkflow(DXDataObject):
                 exec_id = get_app_from_path(exec_id)['id']
 
             executable = get_handler(exec_id)
-            job_name = executable.describe()['title']
+            executable_desc = executable.describe()
+
+            if exec_id.startswith('app-'):
+                stage['app'] = {
+                    "$dnanexus_link": 'app-' + executable_desc['name'] + '/' + executable_desc['version']
+                }
+
+            job_name = executable_desc.get('title', '')
+            if job_name == '':
+                job_name = executable_desc['name']
             job_name += ' - ' + (name if name is not None else workflow_name)
 
             exec_inputs = ExecutableInputs(executable, input_name_prefix=str(stage['key'])+".")
@@ -119,4 +128,7 @@ class DXWorkflow(DXDataObject):
                                                         name=job_name,
                                                         **kwargs)
 
+        # Update workflow with updated executable IDs
+        self.set_details(workflow_spec)
+
         return launched_jobs.values()
