@@ -29,9 +29,10 @@ from datetime import datetime
 import dxpy, dxpy.app_builder
 from dxpy import logger
 
+from dxpy.utils import json_load_raise_on_duplicates
 from dxpy.utils.resolver import resolve_path, is_container_id
 from dxpy.app_categories import APP_CATEGORIES
-from dxpy.exceptions import err_exit
+from dxpy.exceptions import err_exit, DXError
 from dxpy.utils.printing import BOLD
 
 parser = argparse.ArgumentParser(description="Uploads a DNAnexus App.")
@@ -336,6 +337,7 @@ def _verify_app_source_dir(src_dir, enforce=True):
     if "runSpec" in manifest:
         if "interpreter" not in manifest['runSpec']:
             raise dxpy.app_builder.AppBuilderException('runSpec.interpreter field was not present')
+
         if manifest['runSpec']['interpreter'] in ["python2.7", "bash"]:
             if "file" in manifest['runSpec']:
                 entry_point_file = os.path.abspath(os.path.join(src_dir, manifest['runSpec']['file']))
@@ -349,6 +351,19 @@ def _verify_app_source_dir(src_dir, enforce=True):
                 except subprocess.CalledProcessError:
                     raise dxpy.app_builder.AppBuilderException('Code in runSpec.code has syntax errors, see above for details. Rerun with --no-check-syntax to proceed anyway.')
 
+        if 'execDepends' in manifest['runSpec']:
+            if not isinstance(manifest['runSpec']['execDepends'], list):
+                raise dxpy.app_builder.AppBuilderException('Expected runSpec.execDepends to be an array. Rerun with --no-check-syntax to proceed anyway.')
+            if not all(isinstance(dep, dict) for dep in manifest['runSpec']['execDepends']):
+                raise dxpy.app_builder.AppBuilderException('Expected runSpec.execDepends to be an array of hashes. Rerun with --no-check-syntax to proceed anyway.')
+            if any(dep.get('package_manager', 'apt') != 'apt' for dep in manifest['runSpec']['execDepends']):
+                if not isinstance(manifest.get('access'), dict) or 'network' not in manifest['access']:
+                    msg = '\n'.join(['runSpec.execDepends specifies non-APT dependencies, but no network access spec is given.',
+                    'Add {"access": {"network": ["*"]}} to allow dependencies to install.',
+                    'See https://wiki.dnanexus.com/Developer-Tutorials/Request-Additional-App-Resources#Network-Access.',
+                    'Rerun with --no-check-syntax to proceed anyway.'])
+                    raise dxpy.app_builder.AppBuilderException(msg)
+
     # Check all other files that are going to be in the resources tree.
     # For these we detect the language based on the filename extension.
     # Obviously this check can have false positives, since the app can
@@ -378,6 +393,33 @@ def _verify_app_source_dir(src_dir, enforce=True):
         files_str = files_with_problems[0] if len(files_with_problems) == 1 else (files_with_problems[0] + " and " + str(len(files_with_problems) - 1) + " other file" + ("s" if len(files_with_problems) > 2 else ""))
         logging.warn('%s contained syntax errors, see above for details' % (files_str,))
 
+def _verify_app_writable(app_name):
+    app_name_already_exists = True
+    try:
+        current_developers = dxpy.api.app_list_developers('app-' + app_name)['developers']
+    except dxpy.exceptions.DXAPIError as e:
+        if e.name == 'ResourceNotFound':
+            app_name_already_exists = False
+        else:
+            raise e
+
+    if not app_name_already_exists:
+        # This app doesn't exist yet so its creation will succeed
+        # (or at least, not fail on the basis of the ACL).
+        return
+
+    try:
+        me = dxpy.user_info()['userId']
+    except DXError:
+        # Not being able to contact the auth server to do this check
+        # is a non fatal error, since this scenario is common in
+        # testing and the API call to build the app will eventually
+        # fail anyway.
+        print >> sys.stderr, "Warning: could not contact auth server to determine current username."
+        return
+    if me not in current_developers:
+        raise dxpy.app_builder.AppBuilderException('User {user} is not a listed developer for app {app}'.format(user=me, app=app_name))
+
 def _parse_app_spec(src_dir):
     """Returns the parsed contents of dxapp.json.
 
@@ -390,7 +432,7 @@ def _parse_app_spec(src_dir):
         raise dxpy.app_builder.AppBuilderException("Directory %s does not contain dxapp.json: not a valid DNAnexus app source directory" % src_dir)
     with open(os.path.join(src_dir, "dxapp.json")) as app_desc:
         try:
-            return json.load(app_desc)
+            return json_load_raise_on_duplicates(app_desc)
         except Exception as e:
             raise dxpy.app_builder.AppBuilderException("Could not parse dxapp.json file as JSON: " + e.message)
 
@@ -542,7 +584,18 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
             app_run_result = dxpy.api.app_run(builder_app, input_params=api_options)
             job_id = app_run_result["id"]
             print "Started builder job %s" % (job_id,)
-            subprocess.check_call(["dx", "watch", job_id])
+            try:
+                subprocess.check_call(["dx", "watch", job_id])
+            except subprocess.CalledProcessError as e:
+                if e.returncode == 3:
+                    # Some kind of failure to build the app. The reason
+                    # for the failure is probably self-evident from the
+                    # job log (and if it's not, the CalledProcessError
+                    # is not informative anyway), so just propagate the
+                    # return code without additional remarks.
+                    sys.exit(3)
+                else:
+                    raise e
         finally:
             if not using_temp_project_for_remote_build:
                 dxpy.DXProject(build_project_id).remove_objects([remote_file.get_id()])
@@ -563,6 +616,8 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
     app_json = _parse_app_spec(src_dir)
 
     _verify_app_source_dir(src_dir, enforce=do_check_syntax)
+    if mode == "app" and not dry_run:
+        _verify_app_writable(app_json['name'])
 
     working_project = None
     using_temp_project = False
@@ -756,13 +811,14 @@ def main(**kwargs):
         # REMOTE BUILD
 
         try:
-            _parse_app_spec(args.src_dir)
+            app_json = _parse_app_spec(args.src_dir)
+            _verify_app_source_dir(args.src_dir)
+            if args.mode == "app" and not args.dry_run:
+                _verify_app_writable(app_json['name'])
         except dxpy.app_builder.AppBuilderException as e:
             print >> sys.stderr, "Error: %s" % (e.message,)
             sys.exit(3)
 
-        _verify_app_source_dir(args.src_dir)
-
         # The following flags might be useful in conjunction with
         # --remote. To enable these, we need to learn how to pass these
         # options through to the interior call of dx_build_app(let).
