@@ -22,6 +22,7 @@ dx for tab-completion, resolving naming conflicts, etc.
 import sys
 
 from argcomplete import warn
+from collections import namedtuple, OrderedDict
 import dxpy
 from dxpy.utils.resolver import (get_first_pos_of_char, get_last_pos_of_char, clean_folder_path, resolve_path,
                                  split_unescaped, ResolutionError)
@@ -399,3 +400,25 @@ class MultiCompleter():
             return self.matches[state]
         else:
             return None
+
+class InstanceTypesCompleter():
+    InstanceTypeSpec = namedtuple('InstanceTypeSpec', ('Name', 'Memory_GB', 'CPU_Cores', 'Scratch_Space_GB'))
+    instance_types = OrderedDict()
+    for i in (InstanceTypeSpec('dx_m1.medium', 3.75, 1, 420),
+              InstanceTypeSpec('dx_m1.large', 7.5, 2, 840),
+              InstanceTypeSpec('dx_m1.xlarge', 15, 4, 1680),
+              InstanceTypeSpec('dx_c1.xlarge', 7, 8, 1680),
+              InstanceTypeSpec('dx_m2.xlarge', 17.1, 2, 420),
+              InstanceTypeSpec('dx_m2.2xlarge', 34.2, 4, 850),
+              InstanceTypeSpec('dx_m2.4xlarge', 68.4, 8, 1680),
+              InstanceTypeSpec('dx_cc2.8xlarge', 60.5, 32, 3360),
+              InstanceTypeSpec('dx_cg1.4xlarge', 22.5, 16, 1680),
+              InstanceTypeSpec('dx_cr1.8xlarge', 244, 32, 240)):
+        instance_types[i.Name] = i
+    instance_type_names = instance_types.keys()
+
+    def complete(self, text, state):
+        try:
+            return self.instance_type_names[state]
+        except IndexError:
+            return None
