@@ -30,11 +30,7 @@ class DXTestCase(unittest.TestCase):
 
     def tearDown(self):
         try:
-            subprocess.check_call(u"dx rmproject --yes {p}".format(p=self.project), shell=True)
-        except:
-            pass
-        try:
-            os.remove("uploadedfile")
+            subprocess.check_call(u"dx rmproject --yes --quiet {p}".format(p=self.project), shell=True)
         except:
             pass
         if self.old_cwd is not None:
