@@ -220,6 +220,12 @@ class TestDXClient(DXTestCase):
         second_tags = dxpy.describe(second_record_id)['tags']
         self.assertEqual(len(second_tags), 0)
 
+        # nonexistent name
+        with self.assertSubprocessFailure(stderr_regexp='Could not resolve', exit_code=3):
+            run(u"dx tag nonexistent atag")
+        with self.assertSubprocessFailure(stderr_regexp='Could not resolve', exit_code=3):
+            run(u"dx untag nonexistent atag")
+
     def test_dx_project_tagging(self):
         the_tags = [u"$my.tag", u"secoиdtag", u"тhird тagggg"]
         # tag
@@ -234,6 +240,12 @@ class TestDXClient(DXTestCase):
         for tag in [the_tags[0], the_tags[2]]:
             self.assertNotIn(tag, mytags)
 
+        # nonexistent name
+        with self.assertSubprocessFailure(stderr_regexp='Could not find a project named', exit_code=3):
+            run(u"dx tag nonexistent: atag")
+        with self.assertSubprocessFailure(stderr_regexp='Could not find a project named', exit_code=3):
+            run(u"dx untag nonexistent: atag")
+
     def test_dx_object_properties(self):
         property_names = [u"Σ_1^n", u"helloo0", u"ωω"]
         property_values = [u"n", u"world z", u"ω()"]
@@ -271,6 +283,30 @@ class TestDXClient(DXTestCase):
         second_properties = dxpy.api.record_describe(second_record_id, {"properties": True})['properties']
         self.assertEqual(len(second_properties), 0)
 
+        # nonexistent name
+        with self.assertSubprocessFailure(stderr_regexp='Could not resolve', exit_code=3):
+            run(u"dx set_properties nonexistent key=value")
+        with self.assertSubprocessFailure(stderr_regexp='Could not resolve', exit_code=3):
+            run(u"dx unset_properties nonexistent key")
+
+        # Errors parsing --property value
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties -a Ψ ''")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties -a Ψ foo=bar=baz")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties -a Ψ =foo=bar=")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties -a Ψ foo")
+        # Property keys must be nonempty
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx set_properties -a Ψ =bar")
+        # Empty string values should be okay
+        run("dx set_properties -a Ψ bar=")
+
+        my_properties = dxpy.api.record_describe(record_id, {"properties": True})['properties']
+        self.assertEqual(my_properties["bar"], "")
+
     def test_dx_project_properties(self):
         property_names = [u"$my.prop", u"secoиdprop", u"тhird prop"]
         property_values = [u"$hello.world", u"Σ2,n", u"stuff"]
@@ -288,14 +324,155 @@ class TestDXClient(DXTestCase):
         for name in [property_names[0], property_names[2]]:
             self.assertNotIn(name, my_properties)
 
+        # nonexistent name
+        with self.assertSubprocessFailure(stderr_regexp='Could not find a project named', exit_code=3):
+            run(u"dx set_properties nonexistent: key=value")
+        with self.assertSubprocessFailure(stderr_regexp='Could not find a project named', exit_code=3):
+            run(u"dx unset_properties nonexistent: key")
+
+        # Errors parsing --property value
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties : ''")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties : foo=bar=baz")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties : =foo=bar=")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx set_properties : foo")
+        # Property keys must be nonempty
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx set_properties : =bar")
+        # Empty string values should be okay
+        run("dx set_properties : bar=")
+
+        my_properties = dxpy.api.project_describe(self.project, {"properties": True})['properties']
+        self.assertEqual(my_properties["bar"], "")
+
     def test_dx_describe_project(self):
         describe_output = run(u"dx describe :").strip()
         self.assertTrue(re.search(r'ID\s+%s.*\n.*\nName\s+dxclient_test_pr\xc3\xb6ject' % (self.project,),
                                   describe_output))
-        self.assertNotIn('Properties', describe_output)
-        describe_output = run(u"dx describe : --verbose")
         self.assertIn('Properties', describe_output)
 
+    def test_dx_find_data_by_tag(self):
+        record_ids = [run("dx new record --brief --tag Ψ --tag foo --tag baz").strip(),
+                      run("dx new record --brief --tag Ψ --tag foo --tag bar").strip()]
+
+        found_records = run(u"dx find data --tag baz --brief").strip()
+        self.assertEqual(found_records, dxpy.WORKSPACE_ID + ':' + record_ids[0])
+
+        found_records = run(u"dx find data --tag Ψ --tag foo --tag foobar --brief").strip()
+        self.assertEqual(found_records, '')
+
+        found_records = run(u"dx find data --tag foo --tag Ψ --brief").strip().split("\n")
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[0], found_records)
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[1], found_records)
+
+    def test_dx_find_data_by_property(self):
+        record_ids = [run("dx new record --brief --property Ψ=world --property foo=bar --property bar=").strip(),
+                      run("dx new record --brief --property Ψ=notworld --property foo=bar").strip()]
+
+        found_records = run(u"dx find data --property Ψ=world --property foo=bar --brief").strip()
+        self.assertEqual(found_records, dxpy.WORKSPACE_ID + ':' + record_ids[0])
+
+        # presence
+        found_records = run(u"dx find data --property Ψ --brief").strip().split("\n")
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[0], found_records)
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[1], found_records)
+
+        found_records = run(u"dx find data --property Ψ --property foo=baz --brief").strip()
+        self.assertEqual(found_records, '')
+
+        found_records = run("dx find data --property Ψ --property foo=bar --brief").strip().split("\n")
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[0], found_records)
+        self.assertIn(dxpy.WORKSPACE_ID + ':' + record_ids[1], found_records)
+
+        # Empty string values should be okay
+        found_records = run("dx find data --property bar= --brief").strip()
+        self.assertEqual(found_records, dxpy.WORKSPACE_ID + ':' + record_ids[0])
+
+        # Errors parsing --property value
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx find data --property ''")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx find data --property foo=bar=baz")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx find data --property =foo=bar=")
+        # Property keys must be nonempty
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx find data --property =bar")
+
+    def test_dx_find_projects_by_tag(self):
+        other_project_id = run("dx new project other --brief").strip()
+        try:
+            run(u"dx tag : Ψ world")
+            proj_desc = dxpy.describe(dxpy.WORKSPACE_ID)
+            self.assertEqual(len(proj_desc["tags"]), 2)
+            self.assertIn(u"Ψ", proj_desc["tags"])
+            self.assertIn("world", proj_desc["tags"])
+
+            found_projects = run(u"dx find projects --tag Ψ --tag world --brief").strip()
+            self.assertEqual(found_projects, dxpy.WORKSPACE_ID)
+
+            found_projects = run(u"dx find projects --tag Ψ --tag world --tag foobar --brief").strip()
+            self.assertEqual(found_projects, '')
+
+            run(u"dx tag other: Ψ world foobar")
+            found_projects = run("dx find projects --tag world --tag Ψ --brief").strip().split("\n")
+            self.assertIn(dxpy.WORKSPACE_ID, found_projects)
+            self.assertIn(other_project_id, found_projects)
+        except:
+            raise
+        finally:
+            run("dx rmproject -y other")
+
+    def test_dx_find_projects_by_property(self):
+        other_project_id = run("dx new project other --brief").strip()
+        try:
+            run(u"dx set_properties : Ψ=world foo=bar bar=")
+            proj_desc = dxpy.api.project_describe(dxpy.WORKSPACE_ID, {"properties": True})
+            self.assertEqual(len(proj_desc["properties"]), 3)
+            self.assertEqual(proj_desc["properties"][u"Ψ"], "world")
+            self.assertEqual(proj_desc["properties"]["foo"], "bar")
+            self.assertEqual(proj_desc["properties"]["bar"], "")
+
+            run(u"dx set_properties other: Ψ=notworld foo=bar")
+
+            found_projects = run(u"dx find projects --property Ψ=world --property foo=bar --brief").strip()
+            self.assertEqual(found_projects, dxpy.WORKSPACE_ID)
+
+            found_projects = run(u"dx find projects --property bar= --brief").strip()
+            self.assertEqual(found_projects, dxpy.WORKSPACE_ID)
+
+            # presence
+            found_projects = run(u"dx find projects --property Ψ --brief").strip().split("\n")
+            self.assertIn(dxpy.WORKSPACE_ID, found_projects)
+            self.assertIn(other_project_id, found_projects)
+
+            found_projects = run(u"dx find projects --property Ψ --property foo=baz --brief").strip()
+            self.assertEqual(found_projects, '')
+
+            found_projects = run("dx find projects --property Ψ --property foo=bar --brief").strip().split("\n")
+            self.assertIn(dxpy.WORKSPACE_ID, found_projects)
+            self.assertIn(other_project_id, found_projects)
+        except:
+            raise
+        finally:
+            run("dx rmproject -y other")
+
+        # Errors parsing --property value
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx find projects --property ''")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx find projects --property foo=bar=baz")
+        with self.assertSubprocessFailure(stderr_regexp='property_key', exit_code=3):
+            run("dx find projects --property =foo=bar=")
+        # Property keys must be nonempty
+        with self.assertSubprocessFailure(stderr_regexp='nonempty strings', exit_code=3):
+            run("dx find projects --property =bar")
+        # Empty string values should be okay
+        run("dx find projects --property bar=")
+
     def test_dx_remove_project_by_name(self):
         # TODO: this test makes no use of the DXTestCase-provided
         # project.
@@ -349,6 +526,10 @@ class TestDXClient(DXTestCase):
             run(u"dx upload {fd} -o uploadedfile --wait".format(fd=fd.name))
             run(u"dx download uploadedfile -f")
             run(u"dx download uploadedfile -o -")
+        try:
+            os.remove("uploadedfile")
+        except IOError:
+            pass
 
         second_desc = json.loads(run(u"dx describe gritableimport --json"))
         self.assertEqual(second_desc['types'], ['gri'])
@@ -376,11 +557,6 @@ class TestDXClient(DXTestCase):
             with chdir(tempfile.mkdtemp()):
                 run(u'dx download -r '+os.path.basename(wd))
 
-                # Move coverage files generated by previous command so
-                # they get counted, and don't interfere with the
-                # directory tree comparison.
-                subprocess.check_output("mv .coverage.* {old_dir}".format(old_dir=old_dir), shell=True)
-
                 tree1 = subprocess.check_output("cd {wd}; find .".format(wd=wd), shell=True)
                 tree2 = subprocess.check_output("cd {wd}; find .".format(wd=os.path.basename(wd)), shell=True)
                 self.assertEqual(tree1, tree2)
@@ -596,6 +772,40 @@ class TestDXBuildApp(DXTestCase):
         with self.assertSubprocessFailure(stderr_regexp="Expected runSpec\.execDepends to"):
             run("dx build --json " + app_dir)
 
+    def test_duplicate_keys_in_spec(self):
+        app_spec = {
+            "name": "test_duplicate_keys_in_spec",
+            "dxapi": "1.0.0",
+            "runSpec": {
+                "file": "code.py",
+                "interpreter": "python2.7"
+            },
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0"
+            }
+        spec = json.dumps(app_spec).replace('"file": "code.py"', '"file": "code.py", "file": "code.py"')
+        app_dir = self.write_app_directory("duplicate_keys_in_spec", spec, "code.py")
+        with self.assertSubprocessFailure(stderr_regexp="duplicate key: "):
+            run("dx build --json " + app_dir)
+
+    def test_deps_without_network_access(self):
+        app_spec = {
+            "name": "test_deps_without_network_access",
+            "dxapi": "1.0.0",
+            "runSpec": {
+                "file": "code.py",
+                "interpreter": "python2.7",
+                "execDepends": [{"name": "ddd", "package_manager": "pip"}]
+                },
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0"
+            }
+        app_dir = self.write_app_directory("deps_without_network_access", json.dumps(app_spec), "code.py")
+        with self.assertSubprocessFailure(stderr_regexp="runSpec.execDepends specifies non-APT dependencies, but no network access spec is given"):
+            run("dx build --json " + app_dir)
+
     def test_overwrite_applet(self):
         app_spec = {
             "name": "applet_overwriting",
